#  Name : Chaos.rb
#  Description : Move, scale and rotate selected objects randomly within range.
#  Author : Jan Sandstrom (www.pixero.com) 
#  Date : 8 November 2019  
#  Version 1.0
#  Version 1.0.1 Fixed typo so now Move works as it should.
#  Version 1.0.2 Fixed another small typo.


require 'sketchup.rb'

module JS
	def self.chaos
	
		model = Sketchup.active_model # Open model
		ent = model.entities # All entities in model
		sel = model.selection # Current selection
		
		# Dialog
		@use = "World axis" unless @use
		@move = "Off" unless @move				
		@move_axis = "X" unless @move_axis
		@move_min = 0.mm unless @move_min
		@move_max = 100.mm unless @move_max
		@scale = "On" unless @scale
		@scale_axis = "XYZ" unless @scale_axis
		@scale_min = 0.75 unless @scale_min
		@scale_max = 1.5 unless @scale_max
		@rotate = "On" unless @rotate
		@rot_axis = "Z" unless @rot_axis
		@rot_min = 0 unless @rot_min
		@rot_max = 359 unless @rot_max
		prompts = ["Use:", "Move:", "Move Axis:", "Min:", "Max:", "Scale:", "Scale Axis:", "Min:", "Max:", "Rotate:", "Rotate Axis:", "Min:", "Max:"]
		defaults = [@use, @move, @move_axis, @move_min, @move_max, @scale, @scale_axis, @scale_min, @scale_max, @rotate, @rot_axis, @rot_min, @rot_max]
		drops = ["World axis" '|' "Current axis" '|' "Local axis", "On" '|' "Off", "X" '|' "Y" '|' "Z" '|' "XY" '|' "XZ" '|' "YZ" '|' "XYZ", "", "", "On" '|' "Off", "X" '|' "Y" '|' "Z" '|' "XY" '|' "XZ" '|' "YZ" '|' "XYZ", "", "", "On" '|' "Off", "X" '|' "Y" '|' "Z", "", ""]
		title = "Chaos Settings"
		results = inputbox(prompts, defaults, drops, title)
		return nil unless results
		
		@use = results[0]
		@move = results[1]				
		@move_axis = results[2]
		@move_min = results[3].to_l
		@move_max = results[4].to_l
		@scale = results[5]
		@scale_axis = results[6]
		@scale_min = results[7]
		@scale_max = results[8]
		@rotate = results[9]
		@rot_axis = results[10]
		@rot_min = results[11]
		@rot_max = results[12]
		
		if Sketchup.version.split(".")[0].to_i >= 7
			model.start_operation("chaos",true)
		else
			model.start_operation("chaos")
		end
		
			if @use == ("Current axis")
					cur_origin = Sketchup.active_model.axes.origin
					cur_xaxis = Sketchup.active_model.axes.xaxis
					cur_yaxis = Sketchup.active_model.axes.yaxis
					cur_zaxis = Sketchup.active_model.axes.zaxis
					@cur_axis = Geom::Transformation.axes(cur_origin, cur_xaxis, cur_yaxis, cur_zaxis)
			end

			sel.each do |s| 
			##############################################################
				#Move
				if @move == "On"
				  if (@move_axis == "X")
					@m_vector = Geom::Vector3d.new(rand(@move_min..@move_max), 0, 0)
			
				  elsif (@move_axis == "Y")
					@m_vector = Geom::Vector3d.new(0, rand(@move_min..@move_max), 0)
			
				  elsif (@move_axis == "Z")
					@m_vector = Geom::Vector3d.new(0, 0, rand(@move_min..@move_max))
			
				  elsif (@move_axis == "XY")
					@m_vector = Geom::Vector3d.new(rand(@move_min..@move_max), rand(@move_min..@move_max), 0)
			
				  elsif (@move_axis == "XZ")
					@m_vector = Geom::Vector3d.new(rand(@move_min..@move_max), 0, rand(@move_min..@move_max))
			
				  elsif (@move_axis == "YZ")
					@m_vector = Geom::Vector3d.new(0, rand(@move_min..@move_max), rand(@move_min..@move_max))
					
				  elsif (@move_axis == "XYZ")
					@m_vector = Geom::Vector3d.new(rand(@move_min..@move_max), rand(@move_min..@move_max), rand(@move_min..@move_max))
			
				  end#if
				
				

				  if @use == ("World axis")
					s.transform!(Geom::Transformation.translation(@m_vector)) 
				
				  elsif @use == ("Current axis")
					#Temporarily place the component/Group in World axis
					s.transform!(@cur_axis.inverse)
					#Do the transformation in World space
					s.transform!(Geom::Transformation.translation(@m_vector)) 
					#Reset the position and rotation as it was
					s.transform!(@cur_axis)
				
				  elsif @use == ("Local axis")
					s_trans = s.transformation.clone
					#Temporarily place the component/Group in World axis
					s.transform!(s_trans.inverse)
					#Do the transformation in World space
					s.transform!(Geom::Transformation.translation(@m_vector)) 
					#Reset the position and rotation as it was
					s.transform!(s_trans)
				  end
				end#if
				##############################################################  
				#Scale
				if @scale == "On"
				  if @use == ("World axis")
					random_scale = rand(@scale_min..@scale_max)
					if (@scale_axis == "X")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, rand(@scale_min..@scale_max), 1, 1)) 
					elsif (@scale_axis == "Y")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, rand(@scale_min..@scale_max), 1))
					elsif (@scale_axis == "Z")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, 1, rand(@scale_min..@scale_max)))
					elsif (@scale_axis == "XY")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, 1))
					elsif (@move_axis == "XZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, 1, random_scale))
					elsif (@scale_axis == "YZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, random_scale, random_scale))
					elsif (@scale_axis == "XYZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, random_scale))
					end#if							
				
				  elsif @use == ("Current axis")
					#Temporarily place the component/Group in World axis
					s.transform!(@cur_axis.inverse)
					#Do the scale in World space
					random_scale = rand(@scale_min..@scale_max)
					if (@scale_axis == "X")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, rand(@scale_min..@scale_max), 1, 1)) 
					elsif (@scale_axis == "Y")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, rand(@scale_min..@scale_max), 1))
					elsif (@scale_axis == "Z")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, 1, rand(@scale_min..@scale_max)))
					elsif (@scale_axis == "XY")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, 1))
					elsif (@move_axis == "XZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, 1, random_scale))
					elsif (@scale_axis == "YZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, random_scale, random_scale))
					elsif (@scale_axis == "XYZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, random_scale))
					end#if	
					#Reset the position and rotation as it was
					s.transform!(@cur_axis)
				
				  elsif @use == ("Local axis")
					s_trans = s.transformation.clone
					#Temporarily place the component/Group in World axis
					s.transform!(s_trans.inverse)
					#Do scale in World space
					random_scale = rand(@scale_min..@scale_max)
					if (@scale_axis == "X")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, rand(@scale_min..@scale_max), 1, 1)) 
					elsif (@scale_axis == "Y")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, rand(@scale_min..@scale_max), 1))
					elsif (@scale_axis == "Z")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, 1, rand(@scale_min..@scale_max)))
					elsif (@scale_axis == "XY")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, 1))
					elsif (@move_axis == "XZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, 1, random_scale))
					elsif (@scale_axis == "YZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, 1, random_scale, random_scale))
					elsif (@scale_axis == "XYZ")
						s.transform!(Geom::Transformation.scaling(s.transformation.origin, random_scale, random_scale, random_scale))
					end#if	
					#Reset the position and rotation as it was
					s.transform!(s_trans)
				  end
				end #if
				##############################################################
				#Rotate
				if @rotate == "On"
				  if (@rot_axis == "X")
					@r_vector = Geom::Vector3d.new(1, 0, 0)
			
				  elsif (@rot_axis == "Y")
					@r_vector = Geom::Vector3d.new(0, 1, 0)
			
				  elsif (@rot_axis == "Z")
					@r_vector = Geom::Vector3d.new(0, 0, 1)
			
				  end#if

				  if @use == ("World axis")
					s.transform!(Geom::Transformation.rotation(s.transformation.origin, @r_vector, rand(@rot_min..@rot_max))) 
				
				  elsif @use == ("Current axis")
					#Temporarily place the component/Group in World axis
					s.transform!(@cur_axis.inverse)
					#Do the rotation in World space
					s.transform!(Geom::Transformation.rotation(s.transformation.origin, @r_vector, rand(@rot_min..@rot_max))) 
					#Reset the position and rotation as it was
					s.transform!(@cur_axis)
				
				  elsif @use == ("Local axis")
					s_trans = s.transformation.clone
					#Temporarily place the component/Group in World axis
					s.transform!(s_trans.inverse)
					#Do the rotation in World space
					s.transform!(Geom::Transformation.rotation(s.transformation.origin, @r_vector, rand(@rot_min..@rot_max))) 
					#Reset the position and rotation as it was
					s.transform!(s_trans)
				  end
				end#if
				############################################################
			end

		model.commit_operation
		
	end#def
		
	if( not file_loaded?("Chaos.rb"))
		UI.menu("Extensions").add_item("Chaos") { JS.chaos}		
	end
	@file_loaded = true

end #end module

